/* ccgo: game_walk.cc
 * 
 * Copyright 2002 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "game_walk.hh"
#include "root_proper.hh"
#include "modifi_proper.hh"
#include "prop_node.hh"
#include "score_proper.hh"

using namespace go;

void GameWalk::cook_node(Node * n)
{
	if (n == NULL) return;
	ProperNode * pn = dynamic_cast<ProperNode *>(n);
	if (pn == NULL) { // can't cook non-ProperNode
		for (unsigned i = 0; i < n->get_link_size(); i ++) cook_node(n->get_next(i)); // recursion
		return;
	}
	if (pn->get_proper() != NULL) return; // cooked already
	ModifiProper * ap = new ModifiProper;
	pn->set_proper(* this, ap);
	if (ap->get_modifi() == NULL) {
		pn->set_proper(* this, new Proper);
		delete ap;
		ap = NULL;
	} else ap->get_modifi()->apply(* this);
	for (unsigned i = 0; i < n->get_link_size(); i ++) cook_node(n->get_next(i)); // recursion
	if (ap != NULL) ap->get_modifi()->undo(* this);
}

GameWalk::GameWalk(Node * n) :
	Walk(n),
	Game(),
	started(false)
{
}

void GameWalk::cook()
{
	while (num) Walk::up(); // walk to root of the tree
	if (! pos) return;
	ProperNode * pn = dynamic_cast<ProperNode *>(pos);
	if (! pn) return; // error
	RootProper * rp;
	if (pn->get_proper() == NULL) {
		rp = new RootProper;
		pn->set_proper(* this, rp);
	} else {
		rp = dynamic_cast<RootProper *>(pn->get_proper());
		if (! rp) return;
	}
	set_board_size(rp->get_size());
	set_komi(rp->get_komi());
	set_name(rp->get_game_name());
	set_rule(rule_sgf);
	for (unsigned i = 0; i < pos->get_link_size(); i ++) cook_node(pos->get_next(i));
	started = true;
}

void GameWalk::start()
{
	if (started) return;
	while (num) Walk::up(); // walk to root of the tree
	ProperNode * pn = dynamic_cast<ProperNode *>(pos);
	if (! pn) return; // error
	const RootProper * rp = dynamic_cast<const RootProper *>(pn->get_proper());
	if (! rp) return;
	set_board_size(rp->get_size());
	set_komi(rp->get_komi());
	set_name(rp->get_game_name());
	started = true;
}

GameWalk::~GameWalk()
{
	// std::cerr << "deleting GameWalk" << std::endl;
}

bool GameWalk::up()
{
	if (! started) return false;
	if (num == 0) return false;
	ProperNode * pn = dynamic_cast<ProperNode *>(pos);
	if (! Walk::up()) return false;
	if (pn == NULL) return true;
	const ModifiProper * ap = dynamic_cast<const ModifiProper *>(pn->get_proper());
	if (ap != NULL && ap->get_modifi() != NULL) ap->get_modifi()->undo(* this);
	return true;
}

bool GameWalk::down(unsigned i)
{
	if (! started) return false;
	if (! Walk::down(i)) return false;
	ProperNode * pn = dynamic_cast<ProperNode *>(pos);
	if (pn == NULL) return true;
	if (pn->get_proper() == NULL) cook_node(pos);
	const ModifiProper * ap = dynamic_cast<const ModifiProper *>(pn->get_proper());
	if (ap != NULL && ap->get_modifi() != NULL) ap->get_modifi()->apply(* this);
	return true;
}

const std::string & GameWalk::get_comment() const
{
	static const std::string s("");
	ProperNode * pn = dynamic_cast<ProperNode *>(pos);
	if (pn == NULL) return s;
	if (pn->get_proper() == NULL) return s;
	return pn->get_proper()->get_comment();
}

bool GameWalk::add_move(Move * m)
{
	if (! m->check(* this)) {
		// std::cerr << "move fail check: " << m->as_string() << std::endl;
		delete m;
		return false;
	}
	ModifiProper * ap = new ModifiProper;
	ap->set_modifi(m);
	ProperNode * pn = new ProperNode;
	pn->set_proper(* this, ap);
	get_pos()->set_next(pn);
	return true;
}

bool GameWalk::add_score(Terri * t)
{
	ScoreProper * p = new ScoreProper;
	p->set_terri(* this, t);
	ProperNode * pn = new ProperNode;
	pn->set_proper(* this, p);
	get_pos()->set_next(pn);
	return true;
}
